PyModule

Struct PyModule 

pub struct PyModule(/* private fields */);
Available on crate feature tls only.
Represents a Python module object.

Values of this type are accessed via PyO3’s smart pointers, e.g. as Py<PyModule> or Bound<'py, PyModule>.

For APIs available on module objects, see the PyModuleMethods trait which is implemented for Bound<'py, PyModule>.

As with all other Python objects, modules are first class citizens. This means they can be passed to or returned from functions, created dynamically, assigned to variables and so forth.

Implementations



impl PyModule

pub fn new<'py>( py: Python<'py>, name: &str, ) -> Result<Bound<'py, PyModule>, PyErr>

Creates a new module object with the __name__ attribute set to name. When creating a submodule pass the full path as the name such as top_level.name.

Examples
use pyo3::prelude::*;

Python::attach(|py| -> PyResult<()> {
    let module = PyModule::new(py, "my_module")?;

    assert_eq!(module.name()?, "my_module");
    Ok(())
})?;

pub fn import<'py, N>( py: Python<'py>, name: N, ) -> Result<Bound<'py, PyModule>, PyErr>
where N: IntoPyObject<'py, Target = PyString>,

Imports the Python module with the specified name.

Examples
use pyo3::prelude::*;

Python::attach(|py| {
    let module = PyModule::import(py, "antigravity").expect("No flying for you.");
});

This is equivalent to the following Python expression:

import antigravity

If you want to import a class, you can store a reference to it with PyOnceLock::import.

pub fn from_code<'py>( py: Python<'py>, code: &CStr, file_name: &CStr, module_name: &CStr, ) -> Result<Bound<'py, PyModule>, PyErr>

Creates and loads a module named module_name, containing the Python code passed to code and pretending to live at file_name.

If file_name is empty, it will be set to <string>.

⚠ ️
 Warning: This will compile and execute code. Never pass untrusted code to this function!

Errors

Returns PyErr if:

  • code is not syntactically correct Python.
  • Any Python exceptions are raised while initializing the module.
  • Any of the arguments cannot be converted to CStrings.
Example: bundle in a file at compile time with include_str!:
use pyo3::prelude::*;
use pyo3::ffi::c_str;

// This path is resolved relative to this file.
let code = c_str!(include_str!("../../assets/script.py"));

Python::attach(|py| -> PyResult<()> {
    PyModule::from_code(py, code, c"example.py", c"example")?;
    Ok(())
})?;
Example: Load a file at runtime with std::fs::read_to_string.
use pyo3::prelude::*;
use pyo3::ffi::c_str;
use std::ffi::CString;

// This path is resolved by however the platform resolves paths,
// which also makes this less portable. Consider using `include_str`
// if you just want to bundle a script with your module.
let code = std::fs::read_to_string("assets/script.py")?;

Python::attach(|py| -> PyResult<()> {
    PyModule::from_code(py, CString::new(code)?.as_c_str(), c"example.py", c"example")?;
    Ok(())
})?;