PyFloatMethods

Trait PyFloatMethods 

pub trait PyFloatMethods<'py>: Sealed {
    // Required method
    fn value(&self) -> f64;
}
Available on crate feature tls only.
Implementation of functionality for PyFloat.

These methods are defined for the Bound<'py, PyFloat> smart pointer, so to use method call syntax these methods are separated into a trait, because stable Rust does not yet support arbitrary_self_types.

Required Methods

fn value(&self) -> f64

Gets the value of this float.

Implementors



impl<'py> PyFloatMethods<'py> for Bound<'py, PyFloat>